@@ -3,8 +3,8 @@ import collections
 import itertools
 
 from sqlglot import exp, planner
-from sqlglot.dialects import Dialect
 from sqlglot.executor.context import Context
+from sqlglot.dialects.dialect import Dialect, inline_array_sql
 from sqlglot.executor.env import ENV
 from sqlglot.executor.table import Table
 from sqlglot.generator import Generator
@@ -327,9 +327,12 @@ def _ordered_py(self, expression):
 
 
 class Python(Dialect):
+    escape = "\\"
+
     class Generator(Generator):
         TRANSFORMS = {
             exp.Alias: lambda self, e: self.sql(e.this),
+            exp.Array: inline_array_sql,
             exp.And: lambda self, e: self.binary(e, "and"),
             exp.Cast: _cast_py,
             exp.Column: _column_py,
@@ -343,3 +346,15 @@ class Python(Dialect):
             exp.Ordered: _ordered_py,
             exp.Star: lambda *_: "1",
         }
+
+        def case_sql(self, expression):
+            this = self.sql(expression, "this")
+            chain = self.sql(expression, "default") or "None"
+
+            for e in reversed(expression.args["ifs"]):
+                true = self.sql(e, "true")
+                condition = self.sql(e, "this")
+                condition = f"{this} = ({condition})" if this else condition
+                chain = f"{true} if {condition} else ({chain})"
+
+            return chain
